package ru.dbotthepony.mc.otm.block.entity.decorative

import net.minecraft.core.BlockPos
import net.minecraft.core.HolderLookup
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraft.server.level.ServerLevel
import net.minecraft.sounds.SoundSource
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.level.gameevent.GameEvent
import net.neoforged.neoforge.capabilities.Capabilities
import ru.dbotthepony.mc.otm.block.decorative.CargoCrateBlock
import ru.dbotthepony.mc.otm.block.entity.MatteryDeviceBlockEntity
import ru.dbotthepony.mc.otm.capability.IQuickStackContainer
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.container.slotted.FilteredContainerSlot
import ru.dbotthepony.mc.otm.container.slotted.SlottedContainer
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.core.otmRandom
import ru.dbotthepony.mc.otm.menu.MatteryMenuSlot
import ru.dbotthepony.mc.otm.core.util.BlockLootTableHolder
import ru.dbotthepony.mc.otm.menu.decorative.CargoCrateMenu
import ru.dbotthepony.mc.otm.menu.makeSlots
import ru.dbotthepony.mc.otm.registry.game.MBlockEntities
import ru.dbotthepony.mc.otm.registry.game.MSoundEvents

class CargoCrateBlockEntity(
	p_155229_: BlockPos,
	p_155230_: BlockState
) : MatteryDeviceBlockEntity(MBlockEntities.CARGO_CRATE, p_155229_, p_155230_) {
	private inner class Slot(container: SlottedContainer, slot: Int) : FilteredContainerSlot(container, slot) {
		override fun canAutomationPlaceItem(itemStack: ItemStack): Boolean {
			return super<FilteredContainerSlot>.canAutomationPlaceItem(itemStack) && loot.lootTable == null
		}

		override fun canAutomationTakeItem(desired: Int): Boolean {
			return super.canAutomationTakeItem(desired) && loot.lootTable == null
		}
	}

	val container = SlottedContainer.Builder()
		.add(CAPACITY, ::Slot)
		.onChanged(::setChanged)
		.build()
		.also(::addDroppableContainer)

	init {
		exposeSideless(MatteryCapability.QUICK_STACK_CONTAINER, IQuickStackContainer.Simple(makeSlots(container, ::MatteryMenuSlot)))
	}

	private var interactingPlayers = 0

	override fun dropItems(
		oldBlockState: BlockState,
		level: ServerLevel,
		blockPos: BlockPos,
		newBlockState: BlockState,
		movedByPiston: Boolean
	) {
		unpackLootTable()
		super.dropItems(oldBlockState, level, blockPos, newBlockState, movedByPiston)
	}

	override fun beforeDestroyedByPlayer(level: Level, blockPos: BlockPos, blockState: BlockState, player: Player) {
		unpackLootTable(player)
	}

	val loot = BlockLootTableHolder(::markDirtyFast)

	fun onPlayerOpen() {
		val level = level

		if (interactingPlayers++ == 0 && level != null && !isRemoved && level.getBlockState(blockPos).block is CargoCrateBlock) {
			level.setBlock(blockPos, blockState.setValue(CargoCrateBlock.IS_OPEN, true), Block.UPDATE_CLIENTS)
			level.playSound(null, blockPos, MSoundEvents.CARGO_CRATE_OPEN, SoundSource.BLOCKS, 1f, 0.8f + level.otmRandom.nextFloat() * 0.2f)
			level.gameEvent(GameEvent.CONTAINER_OPEN, blockPos, GameEvent.Context.of(blockState))
		}
	}

	fun onPlayerClose() {
		val level = level

		if (--interactingPlayers == 0 && level != null && !isRemoved && level.getBlockState(blockPos).block is CargoCrateBlock) {
			level.setBlock(blockPos, blockState.setValue(CargoCrateBlock.IS_OPEN, false), Block.UPDATE_CLIENTS)
			level.gameEvent(GameEvent.CONTAINER_CLOSE, blockPos, GameEvent.Context.of(blockState))
		}
	}

	init {
		exposeGlobally(Capabilities.ItemHandler.BLOCK, container)
		savetablesLevel.stateful(::container, INVENTORY_KEY)
	}

	override fun saveLevel(nbt: CompoundTag, registry: HolderLookup.Provider) {
		super.saveLevel(nbt, registry)
		loot.save(nbt, registry)
	}

	override fun loadAdditional(nbt: CompoundTag, registry: HolderLookup.Provider) {
		super.loadAdditional(nbt, registry)
		loot.load(nbt, registry)
	}

	private fun unpackLootTable(ply: Player? = null) {
		loot.fill(level as? ServerLevel ?: return, blockPos, container, ply = ply, blockEntity = this)
		loot.clear()
	}

	override val defaultDisplayName: Component
		get() = MACHINE_NAME

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu? {
		if (loot.lootTable != null && ply.isSpectator)
			return null

		unpackLootTable(ply)
		return CargoCrateMenu(containerID, inventory, this)
	}

	companion object {
		private val MACHINE_NAME = TranslatableComponent("block.overdrive_that_matters.cargo_crate")
		const val CAPACITY = 9 * 6
	}
}
