package ru.dbotthepony.mc.otm.block.entity.matter

import net.minecraft.core.BlockPos
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.state.BlockState
import ru.dbotthepony.kommons.util.getValue
import ru.dbotthepony.kommons.util.setValue
import ru.dbotthepony.kommons.util.value
import ru.dbotthepony.mc.otm.block.entity.MatteryDeviceBlockEntity
import ru.dbotthepony.mc.otm.block.entity.tech.BatteryBankBlockEntity
import ru.dbotthepony.mc.otm.capability.FlowDirection
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.matter.IMatterStorage
import ru.dbotthepony.mc.otm.capability.matter.ProfiledMatterStorage
import ru.dbotthepony.mc.otm.container.slotted.ContainerSlot
import ru.dbotthepony.mc.otm.container.slotted.SlottedContainer
import ru.dbotthepony.mc.otm.core.immutableList
import ru.dbotthepony.mc.otm.core.math.Decimal
import ru.dbotthepony.mc.otm.graph.matter.SimpleMatterNode
import ru.dbotthepony.mc.otm.menu.matter.MatterCapacitorBankMenu
import ru.dbotthepony.mc.otm.registry.game.MBlockEntities

class MatterCapacitorBankBlockEntity(p_155229_: BlockPos, p_155230_: BlockState) : MatteryDeviceBlockEntity(MBlockEntities.MATTER_CAPACITOR_BANK, p_155229_, p_155230_), IMatterStorage {
	var gaugeLevel by syncher.float()
		private set

	val profiled = ProfiledMatterStorage(this)
	val matterNode = SimpleMatterNode(matter = profiled)

	override val canSetMatterLevel: Boolean
		get() = false

	override var storedMatter: Decimal
		get() {
			var summ = Decimal.ZERO

			for (stack in container)
				stack.getCapability(MatteryCapability.MATTER_ITEM)?.let {
					summ += it.storedMatter
				}

			return summ
		}
		set(value) {
			throw UnsupportedOperationException()
		}

	override val maxStoredMatter: Decimal get() {
		var summ = Decimal.ZERO

		for (stack in container)
			stack.getCapability(MatteryCapability.MATTER_ITEM)?.let {
				summ += it.maxStoredMatter
			}

		return summ
	}

	private fun updateGaugeLevel() {
		var stored = Decimal.ZERO
		var maxStored = Decimal.ZERO

		for (stack in container) {
			val cap = stack.getCapability(MatteryCapability.MATTER_ITEM) ?: continue
			stored += cap.storedMatter
			maxStored += cap.maxStoredMatter
		}

		gaugeLevel = stored.percentage(maxStored)
	}

	override fun tick() {
		super.tick()
		updateGaugeLevel()
	}

	override fun receiveMatter(howMuch: Decimal, simulate: Boolean): Decimal {
		if (!howMuch.isPositive)
			return Decimal.ZERO

		@Suppress("NAME_SHADOWING")
		var howMuch = howMuch
		var summ = Decimal.ZERO

		for (stack in container) {
			val it = stack.getCapability(MatteryCapability.MATTER_ITEM) ?: continue
			val diff = it.receiveMatterChecked(howMuch, simulate)
			summ += diff
			howMuch -= diff

			if (howMuch.isZero) {
				break
			}
		}

		if (!simulate && !summ.isZero) {
			markDirtyFast()
		}

		return summ
	}

	override fun extractMatter(howMuch: Decimal, simulate: Boolean): Decimal {
		if (!howMuch.isPositive)
			return Decimal.ZERO

		@Suppress("NAME_SHADOWING")
		var howMuch = howMuch
		var summ = Decimal.ZERO

		for (stack in container) {
			val it = stack.getCapability(MatteryCapability.MATTER_ITEM) ?: continue
			val diff = it.extractMatterChecked(howMuch, simulate)
			summ += diff
			howMuch -= diff

			if (howMuch.isZero) {
				break
			}
		}

		if (!simulate && !summ.isZero) {
			markDirtyFast()
		}

		return summ
	}

	override val matterFlow: FlowDirection
		get() = FlowDirection.BI_DIRECTIONAL

	private inner class Slot(container: SlottedContainer, slot: Int) : ContainerSlot(container, slot) {
		override fun notifyChanged(old: ItemStack) {
			super.notifyChanged(old)

			capacitorStatus[slot].value = item.getCapability(MatteryCapability.MATTER_ITEM) != null
		}

		override fun canAutomationPlaceItem(itemStack: ItemStack): Boolean {
			return super.canAutomationPlaceItem(itemStack) && itemStack.getCapability(MatteryCapability.MATTER_ITEM) != null
		}

		override fun canAutomationTakeItem(desired: Int): Boolean {
			item.getCapability(MatteryCapability.MATTER_ITEM)?.let {
				if (it.storedMatter.isPositive) {
					return false
				}
			}

			return super.canAutomationTakeItem(desired)
		}

		override val maxStackSize: Int
			get() = 1
	}

	private fun containerUpdated() {
		markDirtyFast()
		updateGaugeLevel()
	}

	val container = SlottedContainer.simple(BatteryBankBlockEntity.CAPACITY, ::Slot, ::containerUpdated).also(::addDroppableContainer)
	val itemConfig = ConfigurableItemHandler(inputOutput = container)

	val capacitorStatus = immutableList(BatteryBankBlockEntity.CAPACITY) {
		syncher.boolean(false)
	}

	init {
		savetables.stateful(::container, INVENTORY_KEY)
		savetables.stateful(profiled::savedata, "profiled_matter_storage")
		exposeGlobally(MatteryCapability.MATTER_BLOCK, profiled)
		exposeGlobally(MatteryCapability.MATTER_NODE, matterNode)
	}

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return MatterCapacitorBankMenu(containerID, inventory, this)
	}

	override fun setRemoved() {
		super.setRemoved()
		matterNode.isValid = false
	}

	override fun setLevel(level: Level) {
		super.setLevel(level)
		matterNode.discover(this)
	}
}
