package ru.dbotthepony.mc.otm.player

import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.container.ISlottedContainer
import ru.dbotthepony.mc.otm.container.ItemFilter
import ru.dbotthepony.mc.otm.container.get
import ru.dbotthepony.mc.otm.container.set

class PlayerInventoryWrapper(val player: MatteryPlayer) : ISlottedContainer<IPlayerInventorySlot> {
	val inventory: Inventory
		get() = player.ply.inventory

	private inner class Slot(val slot: Int) : IPlayerInventorySlot {
		override fun setChanged() {
			inventory.setChanged()
		}

		override var filter: ItemFilter
			get() = player.slotFilters[slot] ?: ItemFilter.EMPTY
			set(value) { if (value.allowAll) player.slotFilters.remove(slot) else player.slotFilters[slot] = value }
		override var shouldCharge: Boolean
			get() = slot in player.slotsChargeFlag
			set(value) { if (value) player.slotsChargeFlag.add(slot) else player.slotsChargeFlag.remove(slot) }
		override var item: ItemStack
			get() = inventory[slot]
			set(value) { inventory[slot] = value }
		override val maxStackSize: Int
			get() = inventory.maxStackSize

		override fun maxStackSize(item: ItemStack): Int {
			return inventory.getMaxStackSize(item)
		}

		override fun remove(): ItemStack {
			return inventory.removeItemNoUpdate(slot)
		}

		override fun remove(count: Int): ItemStack {
			return inventory.removeItem(slot, count)
		}
	}

	private val slots = Array(SLOTS) { Slot(it) }

	override val hasFilterableSlots: Boolean
		get() = true

	override fun containerSlot(slot: Int): IPlayerInventorySlot {
		return slots[slot]
	}

	override fun clearContent() {
		slots.forEach { it.remove() }
	}

	override fun setChanged() {
		inventory.setChanged()
	}

	override fun getContainerSize(): Int {
		return slots.size
	}

	override fun stillValid(player: Player): Boolean {
		return inventory.stillValid(player)
	}

	companion object {
		const val SLOTS = 41
	}
}
